#
# Monitoring Suite, Seven Solutions S.L.
#
# This file is part of monitoring-7s.
# You might use, distribute and modify this code under the terms of
# the LICENSE.txt provided within the project monitoring-7s.
#
# If you do not have received a copy of the LICENSE.txt along with
# this file please write to info@sevensols.com and consider that
# this file can not be copied and/or distributed in any forms.
#
# Created by Eva Lopez Ruiz in June 2021.
#

"""Comunicate with Grafana API."""


import time
import json
from http.client import HTTPConnection
from base64 import b64encode
from wrz_mon.utils import LOGGER


"""Parameters to HTTP requests"""
METHODS = ['HEAD', 'POST', 'GET', 'PUT']
HEAD, POST, GET, PUT = METHODS

"""API calls"""
ORGS = '/api/org'
AUTH_KEYS = '/api/auth/keys'
HEALTH = '/api/health'
ADMIN_USER = '/api/users/1'
USERS = '/api/admin/users'
PASSWORD = '/api/user/password'
FOLDERS = '/api/folders'
DASHBOARDS = '/api/dashboards/db'
DATASOURCES = '/api/datasources'


class GrafanaClient():
    """A GrafanaClient object allow communication with Grafana API.

    Attributes:
        url (str): Server address
        token (str): Authentication key
        headers (dict): HTTP header request
    """

    def __init__(self, url, token=None, username='admin', password='admin'):
        """Initialize object."""
        self.url = url
        self.token = token
        self.username = username
        self.password = password

    def update_organization(self, organization):
        """Set orgaization.

        Parameters:
            organization (str): Organization name

        Returns:
            retval (int): 0 on success, otherwise -1
        """
        msg = '{"name": "%s"}' % organization
        response, err = self._request(ORGS, PUT, msg)

        if response is None:
            LOGGER.error('Unable to set organization in Grafana service. Exception: %s' % err)
            return -1

        LOGGER.info('Grafana organization changed')
        return 0

    def update_admin_user(self, username, password, email):
        """Set user
        """
        msg1 = dict()
        msg1['name'] = username
        msg1['email'] = email
        msg1['login'] = username

        response, err = self._request(ADMIN_USER, PUT, json.dumps(msg1), True)
        if response is None:
            LOGGER.error('Unable to set user. Exception: {}'.format(err))
            return -1

        self.username = username

        msg2 = {'oldPassword': self.password, 'newPassword': password}
        response, err = self._request(PASSWORD, PUT, json.dumps(msg2), True)
        if response is None:
            LOGGER.error('Unable to set password. Exception: {}'.format(err))
            return -1

        self.password = password

        LOGGER.info('Grafana user and password updated')
        return 0

    def generate_token(self):
        """Generate auth key.

        Parameters:
            username (str): Grafana username
            password (str): Grafana password

        Returns:
            retval (int): 0 on success, otherwise -1
        """
        msg = '{"name": "wrz_mon", "role": "Admin"}'
        response, err = self._request(AUTH_KEYS, POST, msg, True)
        if response is None:
            LOGGER.error('Unable to generate token. Exception: {}'.format(err))
            return None

        self.token = response['key']

        LOGGER.info('Grafana token generated')
        return self.token

    def create_user(self, user, password):
        """Create a new user account

        Parameters:
            user (str): Username
            password (str): Password

        Returns:
            retval (int): 0 on success, otherwise -1
        """
        msg = dict()
        msg['name'] = user
        msg['login'] = user
        msg['password'] = password
        msg['OrgId'] = 1

        response, err = self._request(USERS, POST, json.dumps(msg), True)
        if response is None:
            LOGGER.error('Unable to create {} user. Exception: {}'.format(user, err))
            return -1

        LOGGER.info('Grafana user created')
        return 0

    def create_folder(self, folder):
        """
        """
        msg = dict()
        msg['title'] = folder
        msg['uid'] = 'Eiu54YF7z'
        response, err = self._request(FOLDERS, POST, json.dumps(msg), True)
        if response is None:
            LOGGER.error('Unable to create {} folder. Exception: {}'.format(folder, err))
            return -1

        LOGGER.info('Folder {} created'.format(folder))
        return response['id']

    def import_dashboard(self, dashboard):
        """Import dashboard to Grafana service.

        Parameters:
            dashboard (json): Dashboard to import

        Returns:
            retval (list): Dashboard ID and version on success, otherwise None
        """
        name = dashboard['dashboard']['title']
        response, err = self._request(DASHBOARDS, POST, json.dumps(dashboard))
        if response is None:
            err = 'Unable to import {0} dashboard. Exception: {1}'.format(name, err)
            LOGGER.error(err)
            return None

        LOGGER.info('Dashboard %s imported to Grafana' % name)
        return response['id'], response['version']

    def create_datasource(self, datasource):
        """Create datasource to Grafana service.

        Parameters:
            datasource (json): Datasource to import

        Returns:
            retval (int): The datasource ID on success, otherwise -1
        """
        response, err = self._request(DATASOURCES, POST, json.dumps(datasource))
        if response is None:
            err = 'Unable to import {0} datasource. Exception: {1}'.format(datasource['name'], err)
            LOGGER.error(err)
            return -1

        LOGGER.info('{} datasource imported'.format(datasource['name']))
        return response['datasource']['id']

    def update_datasource(self, datasource):
        """Update datasource to Grafana service.

        Parameters:
            datasource (json): Datasource to import

        Returns:
            retval (int): 0 on success, otherwise -1
        """
        url = DATASOURCES + '/' + str(datasource['id'])
        response, err = self._request(url, PUT, json.dumps(datasource))
        if response is None:
            err = 'Unable to import {0} datasource. Exception: {1}'.format(datasource['name'], err)
            LOGGER.error(err)
            return -1

        LOGGER.info('{} datasource imported'.format(datasource['name']))
        return 0

    def check_service(self):
        """Check if the service is running. It tries three times.

        Returns:
            retval (int): 0 on success, otherwise -1
        """
        sleep_intervals = [5, 10, 15]

        response, err = self._request(HEALTH, GET)

        if response is None:
            for i in sleep_intervals:

                LOGGER.warn("Grafana service doesn't response. Sleep {} seconds".format(i))
                time.sleep(i)

                response, err = self._request(HEALTH, GET)
                if response is not None:
                    break

        if response is None:
            LOGGER.critical('Failure to connect to Grafana service')
            return -1

        return 0

    def _request(self, api_url='/', method=HEAD, body=None, basic_auth=False):
        """Send HTTP request to server.

        Parameters:
            url (str): Server to send request
            api_url (str): ULR to request
            method (str): HTTP method
            body (str): request's argumets
            headers (dict): Headers

        Returns:
            resp (HTTPResponse object): Server's respose
        """
        headers = {'Accept': 'application/json', 'Content-Type': 'application/json'}

        if basic_auth:
            up_bytes = b64encode(bytes(self.username + ':' + self.password, "utf-8"))
            userAndPass = up_bytes.decode("ascii")
            headers['Authorization'] = 'Basic %s' % userAndPass
        else:
            headers['Authorization'] = 'Bearer {}'.format(self.token)

        response = None
        reason = None

        try:
            conn = HTTPConnection(self.url)
            conn.request(method, api_url, body, headers)
            resp = conn.getresponse()

            if resp.status < 200 or 300 <= resp.status:
                reason = resp.reason
            else:
                response = resp.read().decode()
                if response:
                    response = json.loads(response)

            conn.close()

        except Exception as err:
            return None, err

        return response, reason
