#
# Monitoring Suite, Seven Solutions S.L.
#
# This file is part of monitoring-7s.
# You might use, distribute and modify this code under the terms of
# the LICENSE.txt provided within the project monitoring-7s.
#
# If you do not have received a copy of the LICENSE.txt along with
# this file please write to info@sevensols.com and consider that
# this file can not be copied and/or distributed in any forms.
#
# Created by Eva Lopez Ruiz in July 2021.
#

"""InfluxDB Setup Manager."""


import secrets
import os
from wrz_mon.comunication import InfluxClient
from wrz_mon.configuration import Configuration, INFLUXDB_CONTAINER, DATA_DIR
from wrz_mon.utils import Container
from wrz_mon.utils import LOGGER
from wrz_mon.utils import json_dump_file, json_load_file


CONFIG = Configuration()

"""Paths"""
INFLUXDB_IDS = DATA_DIR + '/influxdb_ids.json'
TASKS_DIR = 'influxdb/tasks'

"""Docker container"""
ICONTAINER = Container(INFLUXDB_CONTAINER)


def influxdb_set_initial_setup():
    """Configure influxDB."""
    url = 'localhost:{0}'.format(CONFIG.influxdb_port)
    influx_client = InfluxClient(url)

    if influx_client.check_service() != 0:
        return -1

    LOGGER.info('InfluxDB service started')

    """InfluxDB generates tokens that has 87 digits"""
    CONFIG.influxdb_token = secrets.token_hex(43)
    CONFIG.influxdb_token = CONFIG.influxdb_token + '=='
    LOGGER.info('InfluxDB token generated')

    CONFIG.write_token(INFLUXDB_CONTAINER)

    ids = influx_client.set_initial_setup(CONFIG.username,
                                          CONFIG.password,
                                          CONFIG.influxdb_token,
                                          CONFIG.organization,
                                          CONFIG.bucket)
    if ids is None:
        return -1

    ids_dict = {'user': ids[0], 'organization': ids[1],
                'bucket': ids[2], 'auth': ids[3]}

    err = json_dump_file(INFLUXDB_IDS, ids_dict)
    if err:
        LOGGER.error('Unable to dump datasource ID. Exception: %s' % err)

    task_files = os.listdir(TASKS_DIR)
    for f in task_files:
        path = os.path.join(TASKS_DIR, f)
        content, err = json_load_file(path)
        if content is None:
            continue
        content['org'] = CONFIG.organization
        content['orgID'] = ids_dict['organization']
        influx_client.create_task(content)

    LOGGER.info('InfluxDB successfully configured')
    return 0


def influxdb_update(new_username=None, new_password=None):
    """Update the configuration in InfluxDB service."""
    ids, e = json_load_file(INFLUXDB_IDS)
    if ids is None:
        LOGGER.error('Unable to load InfluxDB IDs. Exception: %s' % e)
        return

    CONFIG.read_token(INFLUXDB_CONTAINER)

    url = 'localhost:{0}'.format(CONFIG.influxdb_port)
    influx_client = InfluxClient(url, CONFIG.influxdb_token, ids['user'],
                                 ids['organization'], ids['bucket'], ids['auth'])

    if influx_client.check_service() != 0:
        return

    if new_username and new_password:
        influx_client.update_user(new_username, new_password)

    if CONFIG.organization_equals_old() is False:
        influx_client.update_organization(CONFIG.organization)
